--     __           _         _            _   _
--  /\ \ \__ _ _ __| | ____ _| | ___ _ __ | |_(_) ___
-- /  \/ / _` | '__| |/ / _` | |/ _ \ '_ \| __| |/ __|
--/ /\  / (_| | |  |   < (_| | |  __/ |_) | |_| | (__
--\_\ \/ \__,_|_|  |_|\_\__,_|_|\___| .__/ \__|_|\___|
--                                  |_|
-- /Narkaleptic/              narkaleptic@mainelan.net
--          Vehicle Control Unit v2.2.6

AddCSLuaFile( "autorun/client/cl_vcu.lua" )
AddCSLuaFile( "resource/language/vcu_english.txt" )

include( "autorun/shared/sh_vcu.lua" )

--------------------------------------------------------------------------------
--------- Vehicle Control Unit
--------------------------------------------------------------------------------

local VcuNextUpdate

function Vcu_IsOwner( Vcu, Player )
	if ( !IsValidEntity( Vcu ) ) then return false end
	
	--Msg( tostring(Vcu:GetTable():GetPlayerIndex()) .. " == " .. Player:UniqueID() .. "\n" )
	
	if ( Vcu:GetTable():GetPlayerIndex() == Player:UniqueID() ) then
		return true
	elseif ( Vcu:GetTable().pl == Player ) then	
		return true
	else
		return false
	end
end

function Vcu_GetOwner( Vcu )
	if ( !IsValidEntity( Vcu ) ) then return NULL end
	
	local Player = player.GetByUniqueID( Vcu:GetTable():GetPlayerIndex() )
	
	if ( Player == false ) then
		return NULL
	else
		return Player
	end
end

function Vcu_GetAssociatedVcu(entity)
	if (!IsValidEntity(entity)) then return nil end

	local vcus = ents.FindByClass("gmod_vehicle_control")
	for _, vcu in pairs(vcus) do
		if (vcu:GetClass() == "gmod_vehicle_control") then
			if (vcu:GetParent() == entity) then
				return vcu
			end
		end
	end

	return nil
end

function Vcu_TipPlayer( Player, Message, Failure )
	
	if ( !Player|| !Player.IsPlayer || !Player:IsPlayer() ) then
		return false 
	end
	
	umsg.Start( "VcuTipMessage", Player )
		umsg.String( "VCU" .. Message )
		umsg.Bool( Failure or false )
	umsg.End()
	
	return true
	
end

function Vcu_SetAnimation(entity, animation)
	if (animation && animation != "" && animation != "x") then
		entity:GetTable().VcuAnimation = animation
		entity:GetTable().HandleAnimation = Vcu_HandleAnimation
	else
		entity:GetTable().VcuAnimation = nil
		entity:GetTable().HandleAnimation = nil
	end
end

function Vcu_HandleAnimation(pl)
	local vehicle = pl:GetVehicle()

	if (!IsValidEntity(vehicle)) then return -1 end

	if (vehicle:GetTable().VcuAnimation) then
		return pl:LookupSequence(vehicle:GetTable().VcuAnimation)
	end

	local seq = -1
	local class = vehicle:GetClass()

	if ( class == "prop_vehicle_jeep" ) then
		seq = pl:LookupSequence( "drive_jeep" )
	elseif ( class == "prop_vehicle_airboat" ) then
		seq = pl:LookupSequence( "drive_airboat" )
	else
		seq = pl:LookupSequence( "drive_pd" )
	end

	return seq
end

function Vcu_Register(entity)
	// TODO
end

function Vcu_Unregister(entity)
	// TODO
end

function Vcu_Eject(vcu)
	local players = player.GetAll()
	for _, player in pairs(players) do
		local found = false

		local vehicle = player:GetVehicle()

		if ( vehicle && vehicle.IsValid && vehicle:IsValid() ) then
			local plvcu = Vcu_GetAssociatedVcu( vehicle )

			if (plvcu == vcu) then
				Vcu_TipPlayer( Vcu_GetOwner(vcu), "Eject" )
				Vcu_TipPlayer( player, "EjectVictim" )
				
				player:ExitVehicle()
				player:ChatPrint( Vcu_GetOwner(vcu):GetName() .. " has ejected you." )
				
				return true
			end			
		end
	end
	
	Vcu_TipPlayer( Vcu_GetOwner( vcu ), "NotInUse", true )
	return false
end

function Vcu_EjectAll(pl, command, arguments)
	local players = player.GetAll()
	for _, player in pairs(players) do

		local vehicle = player:GetVehicle()
		if ( vehicle && vehicle.IsValid && vehicle:IsValid() ) then
			local vcu = Vcu_GetAssociatedVcu(vehicle)
			if ( IsValidEntity(vcu) ) then
				if ( Vcu_IsOwner( vcu, pl ) ) then
					Vcu_TipPlayer( pl, "Eject" )
					Vcu_TipPlayer( player, "EjectVictim" )
				
					player:ExitVehicle()
					player:ChatPrint( pl:GetName() .. " has ejected you." ) 
				end
			end
		end
	end
end
concommand.Add( "vehicle_control_eject_all", Vcu_EjectAll )

/*---------------------------------------------------------
   Name: Vcu_Spawn
   Description: Spawns or returns a gmod_vehicle_contol
---------------------------------------------------------*/
function Vcu_Spawn(player, trace, find_only)
	if (!trace.Hit) then return end

	-- must be an valid entity
	if (!IsValidEntity(trace.Entity)) then return end

	-- must not be a player, weapon or world
	if (trace.Entity:IsPlayer() || trace.Entity:IsWorld() || trace.Entity:IsWeapon()) then return end

	local ent = Vcu_GetAssociatedVcu(trace.Entity)
	if (IsValidEntity(ent)) then
		if ( Vcu_IsOwner( ent, player ) ) then
			return ent, false
		else
			if ( ent:GetNetworkedString( "owner" ) != "" ) then
				player:ChatPrint( "The VCU attached to this vehicle is owned by " .. ent:GetNetworkedString( "owner" ) )
				Vcu_TipPlayer( player, "NotYours", true )
				return nil
			/*else
				ent:Remove()
				player:ChatPrint("An invalid vehicle control unit was removed.")*/
			end
		end
	end

	if (find_only) then return false end

	if ( !player:CheckLimit( "vcu" ) ) then return nil end

	local SpawnPos = trace.Entity:GetPos()

	ent = ents.Create( "gmod_vehicle_control" )
	ent:SetPos( SpawnPos )
	ent:Spawn()
	ent:SetParent(trace.Entity)
	ent:GetTable().pl = player
	ent:SetPlayer( player )
	ent:Activate()
	
	return ent, true
end

-- run Vcu_Check periodically on each vcu
function Vcu_ThinkHook()
    local time = CurTime()

    if time > (VcuNextUpdate or 0) then
		VcuNextUpdate = time + 0.5

		local vcus = ents.FindByClass("gmod_vehicle_control")

		local players = player.GetAll()
		for _, player in pairs(players) do
			local found = false

			local vcu = nil
			local vehicle = player:GetVehicle()
			if (IsValidEntity(vehicle)) then
				for _, v in pairs(vcus) do
					if (v:GetParent() == vehicle) then
						vcu = v
						found = true
						break
					end
				end
			end

			local plvcu = player.Entity:GetNetworkedEntity("vcu")

			if (found && vcu != plvcu) then
				Vcu_Link(player, vcu)
			elseif (!found && IsValidEntity(plvcu)) then
				Vcu_Unlink(player, plvcu)
			end
		end
    end
end
hook.Add( "Think", "Vcu_Think", Vcu_ThinkHook )

function Vcu_Link(player, vcu)
	player.Entity:GetTable().vcu = vcu
	player.Entity:SetNetworkedEntity("vcu", vcu)
	if (IsValidEntity(vcu)) then
		vcu:MessageListeners("InUse", 1) // TODO -- actually check to see if vehicle is in use
	end
end

function Vcu_Unlink(player, vcu)
	Vcu_DeactivateAll(player, player.Entity:GetNetworkedEntity("vcu"))
	player.Entity:GetTable().vcu = nil
	player.Entity:SetNetworkedEntity("vcu", NULL)
	if (IsValidEntity(vcu)) then
		vcu:MessageListeners("InUse", 0) // TODO -- actually check to see if vehicle is in use by someone else ( there is a 1/2 second window that someone can leave and another person can enter the vehicle and the new user joined the server before the previous user )
	end
end

-- release all keys when exiting vehicle
function Vcu_DeactivateAll( player, vcu )
	if ( !player || !player:IsPlayer() ) then return end
	if ( !IsValidEntity(vcu) ) then return end

	local keys = { IN_FORWARD, IN_BACK, IN_MOVELEFT, IN_MOVERIGHT, IN_JUMP, IN_DUCK, IN_SPEED, IN_WALK, IN_RELOAD, IN_ZOOM, IN_ATTACK, IN_ATTACK2 }
	
	local index = vcu:GetVar( "FounderIndex", 0 )

	for _, key in pairs(keys) do
	
		local numkey, name = Vcu_GetMappedKey( key, vcu )
		
		if ( numkey && numkey >= 0 && vcu[name .. 'IsDown'] ) then
			numpad.Deactivate( player, _, { numkey }, index )
			vcu[name .. 'IsDown'] = nil
		end
		
		vcu:MessageListeners( name, 0 )

	end
end

-- monitor key presses
function Vcu_KeyPress( player, key )
    if ( !player ) then return end

    local vcu = player.Entity:GetTable().vcu
	if ( !IsValidEntity(vcu) ) then return end

    local numkey, name = Vcu_GetMappedKey( key, vcu )
	local index = vcu:GetVar( "FounderIndex", 0 )

    if ( numkey && numkey >= 0 && !vcu[name .. 'IsDown']) then
        numpad.Activate( player, _, {numkey}, index )
		vcu[name .. 'IsDown'] = true
    end
	
	vcu:MessageListeners( name, 1 )
end

hook.Add( "KeyPress", "Vcu_KeyPress", Vcu_KeyPress )

-- monitor key releases
function Vcu_KeyRelease( player, key )
    if ( !player ) then return end

    local vcu = player.Entity:GetTable().vcu
	if ( !IsValidEntity(vcu) ) then return end

    local numkey, name = Vcu_GetMappedKey( key, vcu )
	local index = vcu:GetVar( "FounderIndex", 0 )

    if ( numkey && numkey >= 0 && vcu[name .. 'IsDown']) then
        numpad.Deactivate( player, _, {numkey}, index )
		vcu[name .. 'IsDown'] = nil
    end

	vcu:MessageListeners( name, 0 )
end

hook.Add( "KeyRelease", "Vcu_KeyRelease", Vcu_KeyRelease )

function Vcu_GetMappedKey( key, vcu )
    if (key == IN_FORWARD) then return vcu:GetKey( "forward" ), "Forward" end
    if (key == IN_BACK) then return vcu:GetKey( "backward" ), "Backward" end
    if (key == IN_MOVELEFT) then return vcu:GetKey( "left" ), "Left" end
    if (key == IN_MOVERIGHT) then return vcu:GetKey( "right" ), "Right" end
    if (key == IN_JUMP) then return vcu:GetKey( "jump" ), "Jump" end
    if (key == IN_DUCK) then return vcu:GetKey( "duck" ), "Duck" end
    if (key == IN_SPEED) then return vcu:GetKey( "speed" ), "Speed" end
    if (key == IN_WALK) then return vcu:GetKey( "walk" ), "Walk" end
    if (key == IN_ATTACK) then return vcu:GetKey( "attack1" ), "Attack1" end
    if (key == IN_ATTACK2) then return vcu:GetKey( "attack2" ), "Attack2" end
    if (key == IN_RELOAD) then return vcu:GetKey( "reload" ), "Reload" end
    if (key == IN_ZOOM) then return vcu:GetKey( "zoom" ), "Zoom" end
    return nil
end

if (!IsValidEntity) then
	function IsValidEntity(entity)
		if (entity && entity.IsValid && entity:IsValid()) then
			return true
		end

		return false
	end
end
