--     __           _         _            _   _      
--  /\ \ \__ _ _ __| | ____ _| | ___ _ __ | |_(_) ___ 
-- /  \/ / _` | '__| |/ / _` | |/ _ \ '_ \| __| |/ __|
--/ /\  / (_| | |  |   < (_| | |  __/ |_) | |_| | (__ 
--\_\ \/ \__,_|_|  |_|\_\__,_|_|\___| .__/ \__|_|\___|
--                                  |_|               
-- /Narkaleptic/              narkaleptic@mainelan.net
--          Vehicle Control Unit v2.2.6

AddCSLuaFile("cl_init.lua")
AddCSLuaFile("shared.lua")

include("shared.lua")

local VcuPropertyNames = { "forward", "backward", "left", "right", "jump", "duck", "speed", "walk", "reload", "zoom", "attack1", "attack2" }

/*---------------------------------------------------------
   Name: Initialize
---------------------------------------------------------*/
function ENT:Initialize()
	self.Entity:DrawShadow(false)
	self.Entity:SetSolid(SOLID_NONE)
	self.Entity:SetNoDraw(true)
	self.Entity:SetModel("models/props_c17/computer01_keyboard.mdl")
	self.Listeners = {}
	
	Vcu_Register(self.Entity)
end

/*---------------------------------------------------------
   Name: OnRemove
---------------------------------------------------------*/
function ENT:OnRemove()
	local Parent = self.Entity:GetParent()
	if ( IsValidEntity(Parent) ) then
		Parent:SetNetworkedString( "GModOverlayText", "" )
	end

	Vcu_Unregister(self.Entity)
end

/*---------------------------------------------------------
   Name: Setup
---------------------------------------------------------*/
function ENT:Setup(bindings)	
	self.Entity:SetNetworkedString("owner", self.pl:GetName())
	self.Entity:SetNetworkedString("caption", bindings["caption"])
	self.Entity:SetNetworkedFloat("updated", CurTime() )
	
	self.plyindex = self:GetPlayerIndex()
	
	for _, name in pairs(VcuPropertyNames) do
		local key = bindings[name]
		
		-- OL_SuperNumpad compatible
		if (numpad.GetModifiedKey) then 
			key = numpad.GetModifiedKey(self.pl, key)
		end
		
		-- remember numkey
		self:SetKey( name, key )

		-- remember description
		if ( (bindings[name] != -1) || (bindings["ignore_map"] == "1") ) then
			self.Entity:SetNetworkedString(name, bindings[name .. "_desc"])
		else
			self.Entity:SetNetworkedString(name, "")
		end
	end
	
	local Parent = self.Entity:GetParent()
	if ( IsValidEntity(Parent) ) then
		Parent:SetNetworkedString( "GModOverlayText", bindings["caption"] .. "\n" .. self.pl:GetName() )
	end
end

/*---------------------------------------------------------
   Name: AttachListener
---------------------------------------------------------*/
function ENT:AttachListener(entity)
	self:DetachListener(entity)
	
	if (IsValidEntity(entity)) then
		table.insert(self.Listeners, entity)
		return true
	else
		return false
	end
end

/*---------------------------------------------------------
   Name: DetachListener
---------------------------------------------------------*/
function ENT:DetachListener(entity)
	for k, v in ipairs(self.Listeners) do
		if (entity == v) then
			table.remove(self.Listeners, k)
			return true
		end
	end
	
	return false
end

/*---------------------------------------------------------
   Name: MessageListeners
---------------------------------------------------------*/
function ENT:MessageListeners(key, value)
	if (!self.Listeners) then self.Listeners = {} end
	
	for _, entity in pairs(self.Listeners) do
		if (IsValidEntity(entity)) then
			entity:Listen(self.Entity, key, value)
		else
			entity = nil
		end
	end
end

