--     __           _         _            _   _      
--  /\ \ \__ _ _ __| | ____ _| | ___ _ __ | |_(_) ___ 
-- /  \/ / _` | '__| |/ / _` | |/ _ \ '_ \| __| |/ __|
--/ /\  / (_| | |  |   < (_| | |  __/ |_) | |_| | (__ 
--\_\ \/ \__,_|_|  |_|\_\__,_|_|\___| .__/ \__|_|\___|
--                                  |_|               
-- /Narkaleptic/              narkaleptic@mainelan.net
--          Vehicle Control Unit v2.2.6

TOOL.Category 		= "Input"
TOOL.Name 			= "#Vehicle Control Wire"
TOOL.Command 		= nil
TOOL.ConfigName 	= ""

TOOL.ClientConVar["type"] = "gmod_vcu_wire"

cleanup.Register("vcu_wire")

if (SERVER) then
  CreateConVar("sbox_maxvcu_wire", 4)
end

function TOOL.BuildCPanel(cp)
    cp:AddControl("Header", {Text = "#Tool_vcu_wire_name", Description = "#Tool_vcu_wire_desc"})

	cp:AddControl("Label", {Text = "#VcuWire_About"})
end

function TOOL:RightClick(trace)
	if ( self.vcu ) then
		Vcu_TipPlayer( Player, "Cancelled" )
	end
	self.vcu = nil
	self:SetStage(0)
	
	return false
end

function TOOL:LeftClick(trace)
	local Player = self:GetOwner()

	if (!self.vcu) then
		local vcu = Vcu_GetAssociatedVcu(trace.Entity)
		if (IsValidEntity(vcu)) then
			self.vcu = vcu
			self:SetStage(1)
			
			Vcu_TipPlayer( Player, "Selected" )
			
			return true
		else
			Vcu_TipPlayer( Player, "NotFound", true )
		
			return false
		end		
	else 
	    -- do not attach to players
		if trace.Entity && (trace.Entity:IsPlayer() ) then return false end
		
		-- server-side function
		if CLIENT then return true end
		if !SERVER then return false end
	
	    -- check if player is allowed to create more
		local maxed = !self:GetSWEP():CheckLimit("vcu_wire")
	
		local owner = self:GetOwner()
	    
		local pos = trace.HitPos
		local ang = trace.HitNormal:Angle()
		ang.pitch = ang.pitch + 90
	
		-- create the entity	
		entity = MakeVcuWireBeacon(owner, ang, pos, self.vcu)
		if (!IsValidEntity(entity)) then return false end
		
		entity:SetPos(trace.HitPos - trace.HitNormal * entity:OBBMins().z)
		
		local const = nil
		local nocollide = nil
		
		-- if an entity was hit, weld to it
		if (!IsValidEntity(trace.Entity)) then
			const = constraint.Weld(entity, trace.Entity, 0, trace.PhysicsBone, 0, true)
			nocollide = constraint.NoCollide( entity, trace.Entity, 0, trace.PhysicsBone)
			trace.Entity:DeleteOnRemove(entity)
		end
	
		-- create undo
		undo.Create("vcu_wire")
			undo.AddEntity(entity)
			undo.AddEntity(const)
			undo.AddEntity(nocollide)
			undo.SetPlayer(owner)
		undo.Finish()
	
		-- create cleanup
		owner:AddCleanup( "vcu_wire", entity )
		owner:AddCleanup( "vcu_wire", const )
		owner:AddCleanup( "vcu_wire", nocollide )   

		self.vcu = nil
		self:SetStage(0)		
	end
	
	return true
end

if (SERVER) then
	function MakeVcuWireBeacon( Owner, Ang, Pos, Vcu)
		if ( !Owner:CheckLimit("vcu_wire") ) then return nil end
		
		local Entity = ents.Create("gmod_vcu_wire")
		
		if ( !IsValidEntity( Entity ) ) then return false end
		Entity:SetAngles( Ang )
		Entity:SetPos( Pos )
		Entity:SetPlayer( Owner )
		Entity:Spawn()

		Entity:SetVar("Owner", Owner)
		
		if ( IsValidEntity( Vcu ) ) then
			Vcu:AttachListener( Entity )
		end
		
		Owner:AddCount( "vcu_wire", Entity )
		
		Vcu_TipPlayer( Owner, "WireCreate" )
		
		return Entity
	end
	
	duplicator.RegisterEntityClass("vcu_wire", MakeVcuWireBeacon, "ang", "pos", "vcu")
end
