--     __           _         _            _   _      
--  /\ \ \__ _ _ __| | ____ _| | ___ _ __ | |_(_) ___ 
-- /  \/ / _` | '__| |/ / _` | |/ _ \ '_ \| __| |/ __|
--/ /\  / (_| | |  |   < (_| | |  __/ |_) | |_| | (__ 
--\_\ \/ \__,_|_|  |_|\_\__,_|_|\___| .__/ \__|_|\___|
--                                  |_|               
-- /Narkaleptic/              narkaleptic@mainelan.net
--          Vehicle Control Unit v2.2.6

TOOL.Category 		= "Input"
TOOL.Name 			= "#Vehicle Control"
TOOL.Command 		= nil
TOOL.ConfigName 	= ""

TOOL.ClientConVar["type"] = "gmod_vehicle_control"
TOOL.ClientConVar["model"] = "models/props_c17/computer01_keyboard.mdl"

TOOL.ClientConVar["caption"] = "Vehicle Control Unit"
TOOL.ClientConVar["ignore_map"] = 0
TOOL.ClientConVar["forward"] = 8
TOOL.ClientConVar["forward_desc"] = "Move Forward"
TOOL.ClientConVar["backward"] = 5
TOOL.ClientConVar["backward_desc"] = "Move Backward"
TOOL.ClientConVar["left"] = 4
TOOL.ClientConVar["left_desc"] = "Turn Left"
TOOL.ClientConVar["right"] = 6
TOOL.ClientConVar["right_desc"] = "Turn Right"
TOOL.ClientConVar["jump"] = 0
TOOL.ClientConVar["jump_desc"] = "Raise"
TOOL.ClientConVar["duck"] = 10
TOOL.ClientConVar["duck_desc"] = "Lower"
TOOL.ClientConVar["speed"] = 11
TOOL.ClientConVar["speed_desc"] = "Boost"
TOOL.ClientConVar["walk"] = 12
TOOL.ClientConVar["walk_desc"] = "Brake"
TOOL.ClientConVar["attack1"] = 7
TOOL.ClientConVar["attack1_desc"] = "Fire Primary"
TOOL.ClientConVar["attack2"] = 9
TOOL.ClientConVar["attack2_desc"] = "Fire Secondary"
TOOL.ClientConVar["reload"] = -1
TOOL.ClientConVar["reload_desc"] = "Reload"
TOOL.ClientConVar["zoom"] = -1
TOOL.ClientConVar["zoom_desc"] = "Zoom"
TOOL.ClientConVar["animation"] = ""

local VcuPropertyNames = { "forward", "backward", "left", "right", "jump", "duck", "speed", "walk", "reload", "zoom", "attack1", "attack2" }

cleanup.Register("vehicle_control")

if (SERVER) then
  CreateConVar("sbox_maxvcu", 4)
end

function Vcu_UnMapAll( Player, Command, Arguments )
	for _, v in pairs(VcuPropertyNames) do
		Player:ConCommand("vehicle_control_" .. v .. " -1")
		Player:ConCommand("vehicle_control_" .. v .. '_desc ""')
	end
end
concommand.Add( "Vcu_UnMapAll", Vcu_UnMapAll )

function Vcu_ToolEjectAll( Player, Command, Arguments )
	Player:ConCommand("vehicle_control_eject_all\n")
end
concommand.Add( "Vcu_ToolEjectAll", Vcu_ToolEjectAll )
 
if ( SERVER ) then
	-- Copy settings back into the gun
	function Vcu_CopySettings( Player, Command, Arguments )
		local Trace = util.QuickTrace( Player:GetShootPos(), Player:GetAimVector() * 1000, { Player.Entity } ) 
		-- local Trace = Player:GetEyeTrace() -- Doesn't aim properly
		
		if ( !IsValidEntity(Trace.Entity) ) then
			Vcu_TipPlayer( Player, "NotFound", true )
			return
		end
		
		local Vcu = Vcu_GetAssociatedVcu( Trace.Entity )
		if ( !IsValidEntity(Vcu) ) then
			Vcu_TipPlayer( Player, "NotFound", true )
			return
		end
		
		local VcuTable = Vcu:GetTable()
		
		local value = Vcu:GetNetworkedString("caption") 		
		if ( !value || value == "" ) then
			Player:ConCommand( "vehicle_control_caption " .. '""' )
		else
			Player:ConCommand( "vehicle_control_caption " .. '"' .. string.Replace(value, '"', "'") .. '"' )
		end
		
		for _, key in pairs(VcuPropertyNames) do
			
			local numkey = VcuTable:GetKey( key )
			
			Msg( key .. " = " .. numkey .. "\n" )
			
			-- OL_SuperNumpad friendly
			if ( numpad.GetModifiedKey ) then
				numkey = math.fmod( numkey, 50 )
			end
			
			if ( !numkey || numkey < 0 ) then
				Player:ConCommand( "vehicle_control_" .. key .. " -1" )
			else
				Player:ConCommand( "vehicle_control_" .. key .. " " .. numkey )
			end
			
			local value = Vcu:GetNetworkedString( key )
			if ( !value || value == "" ) then
				Player:ConCommand( "vehicle_control_" .. key .. "_desc " .. '""' )
			else
				Player:ConCommand( "vehicle_control_" .. key .. "_desc " .. '"' .. string.Replace(value, '"', "'") .. '"' )
			end
		
		end
		
		Vcu_TipPlayer( Player, "Copied" )
		
		
		return
	end
	concommand.Add( "Vcu_CopySettings", Vcu_CopySettings )
end

function TOOL.BuildCPanel(cp)
    cp:AddControl("Header", {Text = "#Tool_vehicle_control_name", Description = "#Tool_vehicle_control_desc"})

    cp:AddControl("ComboBox", {
        Label = "#Presets",
        MenuButton = "1",
        Folder = "vehicle_control",

        Options = {
            Default = {
            	vehicle_control_caption = "Vehicle Control Unit",
                vehicle_control_ignore_map = 0,
                vehicle_control_forward_desc = "Move Forward",
                vehicle_control_forward = 8,
                vehicle_control_backward_desc = "Move Backward",
                vehicle_control_backward = 5,
                vehicle_control_left_desc = "Turn Left",
                vehicle_control_left = 4,
                vehicle_control_right_desc = "Turn Right",
                vehicle_control_right = 6,
                vehicle_control_jump_desc = "Raise",
                vehicle_control_jump = 0,
                vehicle_control_duck_desc = "Lower",
                vehicle_control_duck = 10,
                vehicle_control_speed_desc = "Boost",
                vehicle_control_speed = 1,
                vehicle_control_walk_desc = "Brake",
                vehicle_control_walk = 3,
                vehicle_control_attack1_desc = "Fire Primary",
                vehicle_control_attack1 = 7,
                vehicle_control_attack2_desc = "Fire Secondary",
                vehicle_control_attack2 = 9,
                vehicle_control_reload_desc = "Reload",
                vehicle_control_reload = -1,
                vehicle_control_zoom_desc = "Zoom",
                vehicle_control_zoom = -1
            }
        },
        CVars = {
			[0] = "vehicle_control_caption",
			[1] = "vehicle_control_ignore_map",
			[2] = "vehicle_control_forward_desc",
			[3] = "vehicle_control_forward",
            [4] = "vehicle_control_backward_desc",
			[5] = "vehicle_control_backward",
            [6] = "vehicle_control_left_desc",
			[7] = "vehicle_control_left",
            [8] = "vehicle_control_right_desc",
			[9] = "vehicle_control_right",
            [10] = "vehicle_control_jump_desc",
			[11] = "vehicle_control_jump",
            [12] = "vehicle_control_duck_desc",
			[13] = "vehicle_control_duck",
            [14] = "vehicle_control_speed_desc",
			[15] = "vehicle_control_speed",
            [16] = "vehicle_control_walk_desc",
			[17] = "vehicle_control_walk",
            [18] = "vehicle_control_attack1_desc",
			[19] = "vehicle_control_attack1",
            [20] = "vehicle_control_attack2_desc",
			[21] = "vehicle_control_attack2",
			[22] = "vehicle_control_reload",
			[23] = "vehicle_control_reload_desc",
			[24] = "vehicle_control_zoom",
			[25] = "vehicle_control_zoom_desc"
		}
	})

	cp:AddControl( "Label", { Text = "#Vcu_Label_0" } )
	cp:AddControl( "Label", { Text = "#Vcu_Label_1" } )
	cp:AddControl( "Button", { Text = "#Vcu_UnmapAll", Description = "#Vcu_UnmapAll_Desc", Command = "Vcu_UnMapAll" } )
	cp:AddControl( "Button", { Text = "#Vcu_EjectAll", Description = "#Vcu_EjectAll_Desc", Command = "Vcu_ToolEjectAll" } )
	cp:AddControl( "Button", { Text = "#Vcu_CopySettings", Description = "#Vcu_CopySettings_Desc", Command = "Vcu_CopySettings" } )
    cp:AddControl( "TextBox", { Label = "#Vcu_Description", Command = "vehicle_control_caption", MaxLength = "22" } )   
    cp:AddControl( "CheckBox", { Label = "#Vcu_IgnoreMap", Description = "#Vcu_IgnoreMap_Desc", Command = "vehicle_control_ignore_map" } )
    /*
	cp:AddControl( "TextBox", { Label = "#Vcu_Forward", Command = "vehicle_control_forward_desc", MaxLength = "22" } )
    cp:AddControl( "Numpad",  { Label = "#Vcu_Forward", Command = "vehicle_control_forward", Buttonsize = "22" } )
    cp:AddControl( "TextBox", { Label = "#Vcu_Backward", Command = "vehicle_control_backward_desc", MaxLength = "22" } )
    cp:AddControl( "Numpad",  { Label = "#Vcu_Backward", Command = "vehicle_control_backward", Buttonsize = "22" } )
	*/

	cp:AddControl( "Label", { Text = "" } )
    cp:AddControl( "TextBox", { Label = "#Vcu_Forward", Command = "vehicle_control_forward_desc", MaxLength = "22" } )
    cp:AddControl( "TextBox", { Label = "#Vcu_Backward", Command = "vehicle_control_backward_desc", MaxLength = "22" } )
    cp:AddControl( "Numpad",  { Label = "#Vcu_Forward", Command = "vehicle_control_forward",
								Label2 = "#Vcu_Backward", Command2 = "vehicle_control_backward", Buttonsize = "22" } )

	cp:AddControl( "Label", { Text = "" } )
	
    cp:AddControl( "TextBox", { Label = "#Vcu_MoveLeft", Command = "vehicle_control_left_desc", MaxLength = "22" } )
    cp:AddControl( "TextBox", { Label = "#Vcu_MoveRight", Command = "vehicle_control_right_desc", MaxLength = "22" } )
    cp:AddControl( "Numpad",  { Label = "#Vcu_MoveLeft", Command = "vehicle_control_left", 
								Label2 = "#Vcu_MoveRight", Command2 = "vehicle_control_right", Buttonsize = "22" } )
    
	cp:AddControl( "Label", { Text = "" } )
	
	cp:AddControl( "TextBox", { Label = "#Vcu_Jump", Command = "vehicle_control_jump_desc", MaxLength = "22" } )
    cp:AddControl( "TextBox", { Label = "#Vcu_Duck", Command = "vehicle_control_duck_desc", MaxLength = "22" } )
    cp:AddControl( "Numpad",  { Label = "#Vcu_Jump", Command = "vehicle_control_jump", 
								Label2 = "#Vcu_Duck", Command2 = "vehicle_control_duck", Buttonsize = "22" } )

	cp:AddControl( "Label", { Text = "" } )
								
    cp:AddControl( "TextBox", { Label = "#Vcu_Speed", Command = "vehicle_control_speed_desc", MaxLength = "22" } )
    cp:AddControl( "TextBox", { Label = "#Vcu_Walk", Command = "vehicle_control_walk_desc", MaxLength = "22" } )
    cp:AddControl( "Numpad",  { Label = "#Vcu_Speed", Command = "vehicle_control_speed",
								Label2 = "#Vcu_Walk", Command2 = "vehicle_control_walk", Buttonsize = "22" } )

								cp:AddControl( "Label", { Text = "" } )
								
    cp:AddControl( "TextBox", { Label = "#Vcu_Attack1", Command = "vehicle_control_attack1_desc", MaxLength = "22" } )
    cp:AddControl( "TextBox", { Label = "#Vcu_Attack2", Command = "vehicle_control_attack2_desc", MaxLength = "22" } )
    cp:AddControl( "Numpad",  { Label = "#Vcu_Attack1", Command = "vehicle_control_attack1", 
								Label2 = "#Vcu_Attack2", Command2 = "vehicle_control_attack2", Buttonsize = "22" } )	
	
	cp:AddControl( "Label", { Text = "" } )

    cp:AddControl( "TextBox", { Label = "#Vcu_Reload", Command = "vehicle_control_reload_desc", MaxLength = "22" } )
    cp:AddControl( "TextBox", { Label = "#Vcu_Zoom", Command = "vehicle_control_zoom_desc", MaxLength = "22" } )
    cp:AddControl( "Numpad",  { Label = "#Vcu_Reload", Command = "vehicle_control_reload", 
								Label2 = "#Vcu_Zoom", Command2 = "vehicle_control_zoom", Buttonsize = "22" } )
	
    /*cp:AddControl( "ComboBox", {
        Label = "Animation",

        Options = {
			Default =  	{ vehicle_control_animation = "x"},
			Jeep =  	{ vehicle_control_animation = "drive_jeep"},
			Airboat = 	{ vehicle_control_animation = "drive_airboat"},
			Pod = 		{ vehicle_control_animation = "drive_pd"}
		},
		CVars = {
			[0] = "vehicle_control_animation"
		},
		Command = "vehicle_control_animation"
	})*/

	cp:AddControl( "Label", { Text = "#Vcu_About" } )
end

function TOOL:Reload( trace )
	
	-- server-side function
	if CLIENT then return true end
	if !SERVER then return false end

	local Owner = self:GetOwner()
	
	local vcu = Vcu_GetAssociatedVcu( trace.Entity )
	if ( !vcu || !vcu.IsValid || !vcu:IsValid() ) then 
		Vcu_TipPlayer( Owner, "NotFound", true )
		return false
	end 
	
	if ( Vcu_IsOwner( vcu, Owner ) ) then
		Vcu_TipPlayer( Owner, "Remove" )
		vcu:Remove()
		return true
	else
		Vcu_TipPlayer( Owner, "NotYours", true )
		return false
	end
	
end

function TOOL:RightClick(trace)
	-- server-side function
	if CLIENT then return true end
	if !SERVER then return false end

	local owner = self:GetOwner()
	
	local vcu = Vcu_GetAssociatedVcu(trace.Entity)
	if (!vcu || !vcu.IsValid || !vcu:IsValid()) then return end 
	
	if ( Vcu_IsOwner( vcu, owner ) ) then		
		return Vcu_Eject(vcu)				
	end
	
	return false
end

function TOOL:LeftClick(trace)

    -- do not attach to players
	if trace.Entity && (trace.Entity:IsPlayer() ) then return false end
	
	-- server-side function
	if CLIENT then return true end
	if !SERVER then return false end

    -- check if player is allowed to create more
	local maxed = !self:GetSWEP():CheckLimit("vcu")

	local owner = self:GetOwner()
    
    -- load control panel properties
    local bound = (self:GetClientNumber("bound") != 0)	    
	local properties = {}
	
	properties["caption"] = self:GetClientInfo("caption")
	properties["ignore_map"] = self:GetClientInfo("ignore_map")
	for _, v in pairs(VcuPropertyNames) do
		properties[v] = self:GetClientNumber(v)
		properties[v .. "_desc"] = self:GetClientInfo(v .. "_desc")
	end

	//local animation = self:GetClientInfo("animation")
	//Vcu_SetAnimation(trace.Entity, animation)

	local ent, spawned = Vcu_Spawn(owner, trace, maxed)
	
	if (ent) then
		ent:GetTable():Setup(properties)
		
		if (spawned) then
		    -- create undo path
			undo.Create("vehicle_control")
				undo.AddEntity( ent )
				undo.SetPlayer( owner )
			undo.Finish()
	
			owner:AddCleanup( "vehicle_control", ent )
			
			Vcu_TipPlayer( owner, "Attach" )
		else
			Vcu_TipPlayer( owner, "Updated" )
		end
	else
		Vcu_TipPlayer( owner, "Failed", true )
		return false
	end

	return true
end
